// tags.e $Revision: 1.5 $
//
// Wed, 09/24/1997 20:22:49
//
// Purpose:
//
//		This module overloads standard Epsilon tags commands and
// routines to reroute tags processing to tagsnt.exe.  This is
// SIGNIFICANTLY faster then Epsilon's built in tagger (several
// orders of magnitude) and tags more information.
//
//		To capture the tagsnt output generated, give an argument to
// the tag_files command.  Normally, output is not captured.
//
// The modifications to this code are dedicated to the public domain
// with the caveat that Lugaru is welcome to use this within their
// distribution source code which is supplied with Epsilon.  Use it
// any way you want for whatever purposes.
//
//
//	John Kercheval
//  4002 NE 204th St
//  Lake Forest Park, WA  98155-1606
//
//	INTERNET: kercheval@bigfoot.com, johnk@wrq.com
//	Compu$erve: 72450,3702
//
//
// Modification History:
//
//		Wed, 09/24/1997 20:27:20 V1.0 release.  
//

#include <eel.h>
#include <tags.h>

char tags_from_bsc;

init_tag_push(
			  char *ctag_list,
			  char *asmtag_list
			 )
{
	char tmpFile[FNAMELEN];
	
	save_var bufnum;
	
	//
	// Create temporary buffers and build the associated file names
	// for the buffers.
	//
	bufnum = tmp_buf();
	give_session_file(tmpFile);
	build_filename(asmtag_list, "%p", tmpFile);
	strcat(asmtag_list, "-asmtag-.lst");
	filename = asmtag_list;
	
	bufnum = tmp_buf();
	give_session_file(tmpFile);
	build_filename(ctag_list, "%p", tmpFile);
	strcat(ctag_list, "-ctag-.lst");
	filename = ctag_list;
}

add_tag_push_file(
				  char *ctag_list,
				  char *asmtag_list,
				  char *file
				 )
{
	char rel[FNAMELEN];
	char tdir[FNAMELEN];
	char *suffix = get_extension(file);

	save_var bufnum;

	if (tag_relative) {
		put_tag_directory(tdir);
		relative(file, rel, tdir);
	} else {
		strcpy(rel, file);
	}

	//
	// .asm and .inc file are ASM files.  All others are C/C++ files
	// (if only it was really that easy).
	//
	if (!strfcmp(".asm", suffix) ||
		!strfcmp(".inc", suffix)) {
		look_file(asmtag_list);
	} else {
		look_file(ctag_list);
	}
	point = size();
	stuff(rel);
	stuff("\n");
}

do_tag_push(
			char *ctag_list,
			char *asmtag_list
		   )
{
	char commandLine[4*FNAMELEN];
	int originalBuffer = bufnum;
	int deleteBuffer;
	int ctag_size = 0;
	int asmtag_size = 0;
	
	//
	// Save the C list file
	//
	look_file(ctag_list);
	ctag_size = size();
	if (ctag_size) {
		file_write(ctag_list, FILETYPE_AUTO);
	}

	//
	// switch the the ASM list file and delete the C list file
	// buffer.
	//
	deleteBuffer = bufnum;
	look_file(asmtag_list);
	buf_delete(deleteBuffer);

	//
	// Save the ASM list file
	//
	asmtag_size = size();
	if (asmtag_size) {
		file_write(asmtag_list, FILETYPE_AUTO);
	}
	
	//
	// Switch to the original buffer the delete the ASM list file
	// buffer.
	//
	deleteBuffer = bufnum;
	bufnum = originalBuffer;
	buf_delete(deleteBuffer);

	//
	// Do the deed
	//
	strcpy(commandLine, "tagsnt -e -T");
	strcat(commandLine, initial_tag_file);
	if (ctag_size) {
		strcat(commandLine, " -c @");
		strcat(commandLine, ctag_list);
	}
	if (asmtag_size) {
		strcat(commandLine, " -a @");
		strcat(commandLine, asmtag_list);
	}
	if (tag_relative) {
		strcat(commandLine, " -r");
	}
	do_push(commandLine, has_arg, 0);

	//
	// Remove the temporary files
	//
	delete_file(ctag_list);
	delete_file(asmtag_list);
}

//
// Taken from, modified and overloads do_retag_file from tags.e in
// Epsilon sources
//
do_retag_file(file)	/* takes abs pathname, retags it */
char *file;
{
	char ctag_list[FNAMELEN];
	char asmtag_list[FNAMELEN];
	char buf[FNAMELEN];

	init_tag_push(ctag_list, asmtag_list);
	add_tag_push_file(ctag_list, asmtag_list, file);
	do_tag_push(ctag_list, asmtag_list);

	//
	// reload the tags file
	//
	strcpy(buf, initial_tag_file);
	absolute(buf);
	sayput("Loading tags from %s...", buf);
	load_tags(buf);

	say("%s tagged.", file);
}

//
// Taken from, modified and overloads tag_files from tags.e in Epsilon
// sources
//
command tag_files() on cx_tab[ALT('.')]
{
	char pat[FNAMELEN], *s;
	char ctag_list[FNAMELEN];
	char asmtag_list[FNAMELEN];
	char buf[FNAMELEN];
	
	init_tags();
	if (tags_from_bsc)
		error("Must tag files using the BSCMAKE program.");
	get_file(pat, "Add/update tags for files matching", filename);
	iter = 0;
	if (!(s = do_file_match(pat, FM_EXACT | FM_NO_DIRS)))
		error("No matches");

	//
	// Create the tag lists and do the tagging
	//
	init_tag_push(ctag_list, asmtag_list);
	for (; s; s = do_file_match(pat, 0))
		add_tag_push_file(ctag_list, asmtag_list, s);
	do_tag_push(ctag_list, asmtag_list);

	//
	// reload the tags file
	//
	strcpy(buf, initial_tag_file);
	absolute(buf);
	sayput("Loading tags from %s...", buf);
	load_tags(buf);
	
	say("%s tagged.", pat);
}

